/*
 * Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

/*
 * Abstract:
 * 	Declaration of hierarchy information related functionality.
 */

#ifndef _OSM_HIERARCHY_INFO_H_
#define _OSM_HIERARCHY_INFO_H_

#ifdef __cplusplus
#  define BEGIN_C_DECLS extern "C" {
#  define END_C_DECLS   }
#else				/* !__cplusplus */
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif				/* __cplusplus */

BEGIN_C_DECLS
/*
 * Forward references.
 */
struct osm_port;
struct osm_physp;
struct osm_node;
struct osm_sm;

/*
 * Template GUID that describes Hierarchy Info for NDR generation switches and HCA.
 * This template GUID specified in the architecture specification "Hierarchy Information Use-Cases"
 */
#define HIERARCHY_INFO_TEMPLATE_GUID_NDR		3

typedef enum osm_hierarchical_info_template_guid_ndr {
	HI_TEMPLATE_GUID_NDR_RECORD_SELECTOR_SPLIT = 0,
	HI_TEMPLATE_GUID_NDR_RECORD_SELECTOR_PORT,
	HI_TEMPLATE_GUID_NDR_RECORD_SELECTOR_CAGE,
	HI_TEMPLATE_GUID_NDR_RECORD_SELECTOR_ASIC,
	HI_TEMPLATE_GUID_NDR_RECORD_SELECTOR_SLOT,
	HI_TEMPLATE_GUID_NDR_RECORD_SELECTOR_TYPE,
	HI_TEMPLATE_GUID_NDR_RECORD_SELECTOR_BDF
} osm_hierarchical_info_template_guid_ndr_t;


#define OSM_PORT_LABEL_LEN				100

typedef struct osm_hierarchy_info {
	ib_hierarchy_info_t block;
	char port_label[OSM_PORT_LABEL_LEN];
} osm_hierarchy_info_t;

/****f* OpenSM: Hierarchy Information/osm_get_hierarchy_info
* NAME
*	osm_get_hierarchy_info
*
* DESCRIPTION
*	Get the hierarchy info MAD for specific port.
*
* SYNOPSIS
*/
void osm_get_hierarchy_info(IN struct osm_sm * sm, IN struct osm_node * p_node,
			    IN struct osm_port * p_port,
			    IN struct osm_physp * p_physp,
			    IN uint8_t index);
/*
* PARAMETERS
*	p_sm
*		[in] Pointer to an osm_sm_t object.
*
*	p_node
*		[in] Pointer to the parent Node object of this Physical Port.
*
*	p_port
*		[in] Pointer to a pointer to a Port object of this Physical port.
*
*	p_physp
*		[in] Pointer to an osm_physp_t object.
*
* 	index
* 		[in] hierarchy index of the block to get.
*
* RETURN VALUES
*	This function does not return a value.
*
* NOTES
*
* SEE ALSO
*	Port, Physical Port
*********/

END_C_DECLS

#endif
